#include "stdafx.h"
#include "CEFDemo.h"
#include "MainFrm.h"
#include "ChildFrm.h"
#include "CEFDemoDoc.h"
#include "CEFDemoView.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#endif


#pragma warning(suppress: 26440 26433)
BEGIN_MESSAGE_MAP(CCEFDemoApp, CWinApp) //NOLINT(clang-diagnostic-unused-local-typedef, modernize-avoid-c-arrays)
  ON_COMMAND(ID_APP_ABOUT, &CCEFDemoApp::OnAppAbout)
  ON_COMMAND(ID_FILE_NEW, &CWinApp::OnFileNew)
  ON_COMMAND(ID_FILE_OPEN, &CWinApp::OnFileOpen)
END_MESSAGE_MAP()

#pragma warning(suppress: 26426)
CCEFDemoApp theApp;


CCEFDemoApp::CCEFDemoApp() noexcept : m_bDidCefInitialize{false}
{
}

//CEF_INTEGRATION
bool CCEFDemoApp::TryLoadAllImports(_In_z_ LPCSTR szDll) noexcept
{
  __try
  {
    return SUCCEEDED(__HrLoadAllImportsForDll(szDll));
  }
#pragma warning(suppress: 6320)
  __except (EXCEPTION_EXECUTE_HANDLER)
  {
    return false;
  }
}

BOOL CCEFDemoApp::InitInstance()
{
  //Initialize common controls
  INITCOMMONCONTROLSEX InitCtrls{};
  InitCtrls.dwSize = sizeof(InitCtrls);
  InitCtrls.dwICC = ICC_WIN95_CLASSES;
  InitCommonControlsEx(&InitCtrls);

  //For demonstration purposes lets load CEF from a subdirectory called "CEF" below where the main exe is located //CEF_INTEGRATION
  CString sCEFDLLPath;
  GetModuleFileName(AfxGetInstanceHandle(), sCEFDLLPath.GetBuffer(_MAX_PATH), _MAX_PATH);
  sCEFDLLPath.ReleaseBuffer();
  CString sDrive;
  CString sDir;
  CString sFname;
  _tsplitpath_s(sCEFDLLPath, sDrive.GetBuffer(_MAX_DRIVE), _MAX_DRIVE, sDir.GetBuffer(_MAX_DIR), _MAX_DIR, nullptr, 0, nullptr, 0);
  sDir.ReleaseBuffer();
  sDrive.ReleaseBuffer();
  sDir += _T("CEF\\");
  if (!SetDllDirectory(sDir)) //This is required because it seems some of CEF itself loads DLL using relative paths
  {
    const DWORD dwLastError{GetLastError()};
    CString sError;
    sError.Format(_T("Failed to set the DLL search directories to include '%s'. Error: %u"), sDir.GetString(), dwLastError);
    AfxMessageBox(sError);
    return FALSE;
  }
  _tmakepath_s(sCEFDLLPath.GetBuffer(_MAX_PATH), _MAX_PATH, sDrive, sDir, _T("libcef"), _T(".dll"));
  sCEFDLLPath.ReleaseBuffer();
  if (LoadLibrary(sCEFDLLPath) == nullptr)
  {
    AfxMessageBox(_T("Could not load CEF. Please provide the binary files for CEF in a directory called CEF below this application's directory."));
    return FALSE;
  }
  if (!SetDllDirectory(nullptr))
  {
    const DWORD dwLastError{GetLastError()};
    CString sError;
    sError.Format(_T("Failed to restore DLL search directories to default. Error:%u"), dwLastError);
    AfxMessageBox(sError);
    return FALSE;
  }
  if (!TryLoadAllImports("libcef.dll"))
  {
    AfxMessageBox(_T("Failed to resolve imports for CEF. Please provide the binary files for CEF in a directory called CEF below this application's directory."));
    return FALSE;
  }


  //Initialize CEF //CEF_INTEGRATION
  if (!InitializeCef())
    return FALSE;

  SetRegistryKey(_T("PJ Naughter"));
  __super::InitInstance();
  LoadStdProfileSettings(); //Load standard INI file options (including MRU)

  //Register the application's document templates.  Document templates
  //serve as the connection between documents, frame windows and views.
#pragma warning(suppress: 26400 26403 26409)
  auto pDocTemplate{new CMultiDocTemplate(IDR_CEFTYPE, RUNTIME_CLASS(CCEFDemoDoc), RUNTIME_CLASS(CChildFrame), RUNTIME_CLASS(CCEFDemoView))};
  if (pDocTemplate == nullptr)
    return FALSE;
  AddDocTemplate(pDocTemplate);

  //Create main MDI frame window
#pragma warning(suppress: 26400 26403 26409)
  auto pMainFrame{new CMainFrame};
  if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
    return FALSE;
  m_pMainWnd = pMainFrame;

  //Parse command line for standard shell commands, DDE, file open
  CCommandLineInfo cmdInfo;
  ParseCommandLine(cmdInfo);

  //Dispatch commands specified on the command line
  if (!ProcessShellCommand(cmdInfo))
    return FALSE;

  //The main window has been initialized, so show and update it
  m_pMainWnd->ShowWindow(SW_SHOW);
  m_pMainWnd->UpdateWindow();

  return TRUE;
}

int CCEFDemoApp::ExitInstance()
{
  //CEF_INTEGRATION
  if (m_bDidCefInitialize)
    CefShutdown();

  //Let the base class do its thing
  return __super::ExitInstance();
}

//CEF_INTEGRATION
BOOL CCEFDemoApp::PumpMessage()
{
  const auto bResult{__super::PumpMessage()};

  //If there are other messages on queue then return right away
  //otherwise CEF has a habit of eating keystrokes not meant for it
  MSG msg;
  if (PeekMessage(&msg, nullptr, 0, 0, PM_NOREMOVE))
    return bResult;

  //Allow Cef to do his thing
  CefDoMessageLoopWork();

  return bResult;
}

//CEF_INTEGRATION
bool CCEFDemoApp::InitializeCef()
{
  //Initialize Cef
#pragma warning(suppress: 26409)
  const CefMainArgs mainargs{m_hInstance};

  //See if this is a Cef process starting. If so, we're done with our (and MFC) initialization.
  const auto nExitCode{CefExecuteProcess(mainargs, nullptr, nullptr)};
  if (nExitCode >= 0)
    return false;

  CefSettings settings;
  settings.multi_threaded_message_loop = false;
  settings.no_sandbox = true;
  m_bDidCefInitialize = true;
  m_CefApp = new CCEFAppHandler;
  return CefInitialize(mainargs, settings, m_CefApp.get(), nullptr);
}



class CAboutDlg : public CDialog
{
public:
  CAboutDlg();
  CAboutDlg(const CAboutDlg&) = delete;
  CAboutDlg(CAboutDlg&&) = delete;
  ~CAboutDlg() = default; //NOLINT(modernize-use-override)

  CAboutDlg& operator=(const CAboutDlg&) = delete;
  CAboutDlg& operator=(CAboutDlg&&) = delete;

  enum { IDD = IDD_ABOUTBOX };

protected:
  void DoDataExchange(CDataExchange* pDX) override;

  DECLARE_MESSAGE_MAP()
};

#pragma warning(suppress: 26455)
CAboutDlg::CAboutDlg() : CDialog{CAboutDlg::IDD}
{
}

#pragma warning(suppress: 26433)
void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
  //Let the base class do its thing
  __super::DoDataExchange(pDX);
}

#pragma warning(suppress: 26440 26433)
BEGIN_MESSAGE_MAP(CAboutDlg, CDialog) //NOLINT(clang-diagnostic-unused-local-typedef, modernize-avoid-c-arrays)
END_MESSAGE_MAP()

void CCEFDemoApp::OnAppAbout()
{
  CAboutDlg aboutDlg;
  aboutDlg.DoModal();
}
