#pragma once

#include "CEFWrappers.h"
#include "CEFDemoDoc.h"
#include "resource.h"


class CCEFDemoBinaryResourceProvider : public CefResourceManager::Provider
{
public:
//Constructors / Destructors
  CCEFDemoBinaryResourceProvider(_In_ const std::string& sURLPath, _In_ const std::string& sResourcePathPrefix) : m_sURLPath{sURLPath}, //NOLINT(modernize-pass-by-value)
                                                                                                                  m_sResourcePathPrefix{sResourcePathPrefix}
  {
    ASSERT(!m_sURLPath.empty());
#pragma warning(suppress: 26446)
    if (!m_sResourcePathPrefix.empty() && (m_sResourcePathPrefix[m_sResourcePathPrefix.length() - 1] != '/'))
      m_sResourcePathPrefix += "/";
  }
  CCEFDemoBinaryResourceProvider(const CCEFDemoBinaryResourceProvider&) = delete;
  CCEFDemoBinaryResourceProvider(CCEFDemoBinaryResourceProvider&&) = delete;
  virtual ~CCEFDemoBinaryResourceProvider() = default; //NOLINT(modernize-use-override)

//Methods
  CCEFDemoBinaryResourceProvider& operator=(const CCEFDemoBinaryResourceProvider&) = delete;
  CCEFDemoBinaryResourceProvider& operator=(CCEFDemoBinaryResourceProvider&&) = delete;

#pragma warning(suppress: 26415 26418)
  bool OnRequest(scoped_refptr<CefResourceManager::Request> request) override
  {
    CEF_REQUIRE_IO_THREAD();

    const std::string& url{request->url()};
    if (url.find(m_sURLPath) != 0L)
      return false; //Not handled by this provider

    CefRefPtr<CefResourceHandler> handler;
    std::string sRelativePath{url.substr(m_sURLPath.length())};
    if (!sRelativePath.empty())
    {
      if (!m_sResourcePathPrefix.empty())
        sRelativePath = m_sResourcePathPrefix + sRelativePath;
      CefRefPtr<CefStreamReader> stream{GetBinaryResourceReader(sRelativePath.data())};
      if (stream.get())
#pragma warning(suppress: 26409)
        handler = new CefStreamResourceHandler{request->mime_type_resolver().Run(url), stream};
    }
    request->Continue(handler);
    return true;
  }

  CefRefPtr<CefStreamReader> GetBinaryResourceReader(_In_ const std::string& sResourceName)
  {
    const int nResourceId{GetResourceId(sResourceName)};
    if (nResourceId == 0)
    {
      TRACE(_T("Could not find specified resource, name:%s\n"), CString{CefString{sResourceName.c_str()}.ToWString().c_str()}.GetString()); //The resource should be found
      return nullptr;
    }
    DWORD dwSize{0};
    LPBYTE pBytes{nullptr};
    if (LoadBinaryResource(nResourceId, dwSize, pBytes))
#pragma warning(suppress: 26409)
      return CefStreamReader::CreateForHandler(new CefByteReadHandler{pBytes, dwSize, nullptr});
    TRACE(_T("Could not load resource, name:%s, ID:%d\n"), CString{CefString{sResourceName.c_str()}.ToWString().c_str()}.GetString(), nResourceId); //The resource should be found
    return nullptr;
  }

protected:
//Methods
  int GetResourceId(_In_ const std::string& sResourceName)
  {
    //Map of resource labels to BINARY id values
    static std::map<std::string, int> g_ResourceMap
    { {
        {  "intro.html",          IDS_INTRO_HTML          },
        {  "pdf.html",            IDS_PDF_HTML            },
        {  "pdf.pdf",             IDS_PDF_PDF             },
        {  "window_icon.1x.png",  IDS_WINDOW_ICON_1X_PNG  },
        {  "window_icon.2x.png",  IDS_WINDOW_ICON_2X_PNG  }
    } };
    const auto iter{g_ResourceMap.find(sResourceName)};
    if (iter != g_ResourceMap.cend())
      return iter->second;
    else
      return 0;
  }

  bool LoadBinaryResource(_In_ int binaryId, _Inout_ DWORD& dwSize, _Inout_ LPBYTE& pBytes)
  {
    HINSTANCE hInst{AfxGetResourceHandle()};
    HRSRC hRes{FindResource(hInst, MAKEINTRESOURCE(binaryId), MAKEINTRESOURCE(BINARY_RESOURCE_TYPE))};
    if (hRes)
    {
      HGLOBAL hGlob{LoadResource(hInst, hRes)};
      if (hGlob)
      {
        dwSize = SizeofResource(hInst, hRes);
        pBytes = static_cast<LPBYTE>(LockResource(hGlob));
        if ((dwSize > 0) && pBytes)
          return true;
      }
    }
    return false;
  }

//Member variables
  std::string m_sURLPath;
  std::string m_sResourcePathPrefix;
};


class CCEFDemoClientHandler : public CCEFClientHandler
{
public:
//Constructors / Destructors
  CCEFDemoClientHandler() = delete;
  CCEFDemoClientHandler(_In_opt_ CCEFDelegate* pDelegate) : CCEFClientHandler{pDelegate},
#pragma warning(disable: 26409)
                                                            m_ResourceManager{new CefResourceManager{}}
  {
    SetupResourceManager(m_ResourceManager);
  }
  CCEFDemoClientHandler(const CCEFClientHandler&) = delete;
  CCEFDemoClientHandler(CCEFClientHandler&&) = delete;
#pragma warning(disable: 26432)
  ~CCEFDemoClientHandler() = default; //NOLINT(modernize-use-override)

  //Methods
  CCEFDemoClientHandler& operator=(const CCEFDemoClientHandler&) = delete;
  CCEFDemoClientHandler& operator=(CCEFDemoClientHandler&&) = delete;

  CefRefPtr<CefResourceHandler> GetResourceHandler(CefRefPtr<CefBrowser> browser, CefRefPtr<CefFrame> frame, CefRefPtr<CefRequest> request) override
  {
    CEF_REQUIRE_IO_THREAD();
    ASSERT(m_ResourceManager != nullptr);
    return m_ResourceManager->GetResourceHandler(browser, frame, request);
  }

  CefRefPtr<CefRequestHandler> GetRequestHandler() override
  {
    return this;
  }

#pragma warning(disable: 26418)
  CefRefPtr<CefResourceRequestHandler> GetResourceRequestHandler(CefRefPtr<CefBrowser> /*browser*/, CefRefPtr<CefFrame> /*frame*/, CefRefPtr<CefRequest> /*request*/,
                                                                 bool /*is_navigation*/, bool /*is_download*/, const CefString& /*request_initiator*/, bool& /*disable_default_handling*/) override
  {
    CEF_REQUIRE_IO_THREAD();
    return this;
  }

  cef_return_value_t OnBeforeResourceLoad(CefRefPtr<CefBrowser> browser, CefRefPtr<CefFrame> frame, CefRefPtr<CefRequest> request, CefRefPtr<CefCallback> callback) override
  {
    CEF_REQUIRE_IO_THREAD();
    ASSERT(m_ResourceManager != nullptr);
#pragma warning(suppress: 26812)
    return m_ResourceManager->OnBeforeResourceLoad(browser, frame, request, callback);
  }

#pragma warning(suppress: 26415 26418)
  static void SetupResourceManager(_Inout_ CefRefPtr<CefResourceManager> resource_manager)
  {
    if (!CefCurrentlyOn(TID_IO))
    {
      //Execute on the browser IO thread
      CefPostTask(TID_IO, base::BindOnce(SetupResourceManager, resource_manager));
      return;
    }

    //Add the Provider for bundled resource files
    ASSERT(resource_manager != nullptr);
    resource_manager->AddProvider(CreateBinaryResourceProvider("http://tests/", std::string{}), 100, std::string{});
  }

  static CefResourceManager::Provider* CreateBinaryResourceProvider(_In_ const std::string& url_path, _In_ const std::string& resource_path_prefix)
  {
#pragma warning(suppress: 26409)
    return new CCEFDemoBinaryResourceProvider{url_path, resource_path_prefix};
  }

protected:
//Member variables
  CefRefPtr<CefResourceManager> m_ResourceManager;
};


class CCEFDemoView : public CCEFView
{
protected: //create from serialization only
  CCEFDemoView();
  DECLARE_DYNCREATE(CCEFDemoView)

public:
//Methods
  CCEFDemoDoc* GetDocument();

protected:
//Methods
  void OnSetAddress(_Inout_ CefRefPtr<CefBrowser> browser, _Inout_ CefRefPtr<CefFrame> frame, _In_ const std::string& url) override;
  void OnSetTitle(_Inout_ CefRefPtr<CefBrowser> browser, _In_ const std::string& title) override;
  void Toggle(_In_ HWND hWndRoot, _In_ UINT nCmdShow) noexcept;
  void OnFullscreenModeChange(_Inout_ CefRefPtr<CefBrowser> browser, _In_ bool fullscreen) override;
  void OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView) override;
  void OnInitialUpdate() override;

//Member variables
  CString m_sURL;
  WINDOWPLACEMENT m_BeforeFullScreenPlacement;

  DECLARE_MESSAGE_MAP()
};
